#!/usr/bin/env python3

import sys
import os
import time
import argparse
from Bio.PDB import *
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_io.api.pdb import pdb
from biobb_model.model.fix_side_chain import fix_side_chain
from biobb_model.model.mutate import mutate
from biobb_md.gromacs.pdb2gmx import pdb2gmx
from biobb_md.gromacs.editconf import editconf
from biobb_md.gromacs.solvate import solvate
from biobb_md.gromacs.grompp import grompp
from biobb_md.gromacs.genion import genion
from biobb_md.gromacs.mdrun import mdrun
from biobb_analysis.gromacs.gmx_rms import gmx_rms
from biobb_analysis.gromacs.gmx_rgyr import gmx_rgyr
from biobb_analysis.gromacs.gmx_energy import gmx_energy
from biobb_analysis.gromacs.gmx_image import gmx_image
from biobb_analysis.gromacs.gmx_trjconv_str import gmx_trjconv_str

# Loading the biobb configuration reader
conf = settings.ConfReader(sys.argv[1])

global_log, _ = fu.get_logs(path=conf.get_working_dir_path())
global_prop = conf.get_prop_dic(global_log=global_log)
global_paths = conf.get_paths_dic()

global_log.info("step1_pdb: Download the initial Structure")
pdb(**global_paths["step1_pdb"], properties=global_prop["step1_pdb"])

global_log.info("step2_fixsidechain: Modeling the missing heavy atoms in the structure side chains")
fix_side_chain(**global_paths["step2_fixsidechain"], properties=global_prop["step2_fixsidechain"])

scan_residue = 'ALA'

mut_list = [res.get_resname()+str(res.get_id()[1])+scan_residue
for res in Selection.unfold_entities(PDBParser().get_structure(
global_prop["step1_pdb"]["pdb_code"], global_paths["step1_pdb"]["output_pdb_path"]), 'R')]

for mutation in mut_list:

    mut_paths = conf.get_paths_dic(mutation)
    mut_prop = conf.get_prop_dic(mutation, global_log=global_log)

    mut_paths['step3_mutate']['input_pdb_path'] = global_paths['step2_fixsidechain']['output_pdb_path']

    global_log.info("step3_mutate: Modeling a particular residue mutation")
    mutate(**mut_paths["step3_mutate"], properties=mut_prop["step3_mutate"])

    global_log.info("step4_pdb2gmx: Generate the topology")
    pdb2gmx(**mut_paths["step4_pdb2gmx"], properties=mut_prop["step4_pdb2gmx"])

    global_log.info("step5_editconf: Create the solvent box")
    editconf(**mut_paths["step5_editconf"], properties=mut_prop["step5_editconf"])

    global_log.info("step6_solvate: Fill the solvent box with water molecules")
    solvate(**mut_paths["step6_solvate"], properties=mut_prop["step6_solvate"])

    global_log.info("step7_grompp_genion: Preprocess ion generation")
    grompp(**mut_paths["step7_grompp_genion"], properties=mut_prop["step7_grompp_genion"])

    global_log.info("step8_genion: Ion generation")
    genion(**mut_paths["step8_genion"], properties=mut_prop["step8_genion"])

    global_log.info("step9_grompp_min: Preprocess energy minimization")
    grompp(**mut_paths["step9_grompp_min"], properties=mut_prop["step9_grompp_min"])

    global_log.info("step10_mdrun_min: Execute energy minimization")
    mdrun(**mut_paths["step10_mdrun_min"], properties=mut_prop["step10_mdrun_min"])

    global_log.info("step11_energy_min: Compute potential energy during minimization")
    gmx_energy(**mut_paths["step11_energy_min"], properties=mut_prop["step11_energy_min"])

    global_log.info("step12_grompp_nvt: Preprocess system temperature equilibration")
    grompp(**mut_paths["step12_grompp_nvt"], properties=mut_prop["step12_grompp_nvt"])

    global_log.info("step13_mdrun_nvt: Execute system temperature equilibration")
    mdrun(**mut_paths["step13_mdrun_nvt"], properties=mut_prop["step13_mdrun_nvt"])

    global_log.info("step14_energy_nvt: Compute temperature during NVT equilibration")
    gmx_energy(**mut_paths["step14_energy_nvt"], properties=mut_prop["step14_energy_nvt"])

    global_log.info("step15_grompp_npt: Preprocess system pressure equilibration")
    grompp(**mut_paths["step15_grompp_npt"], properties=mut_prop["step15_grompp_npt"])

    global_log.info("step16_mdrun_npt: Execute system pressure equilibration")
    mdrun(**mut_paths["step16_mdrun_npt"], properties=mut_prop["step16_mdrun_npt"])

    global_log.info("step17_energy_npt: Compute Density & Pressure during NPT equilibration")
    gmx_energy(**mut_paths["step17_energy_npt"], properties=mut_prop["step17_energy_npt"])

    global_log.info("step18_grompp_md: Preprocess free dynamics")
    grompp(**mut_paths["step18_grompp_md"], properties=mut_prop["step18_grompp_md"])

    global_log.info("step19_mdrun_md: Execute free molecular dynamics simulation")
    mdrun(**mut_paths["step19_mdrun_md"], properties=mut_prop["step19_mdrun_md"])

    global_log.info("step20_rmsfirst: Compute Root Mean Square deviation against equilibrated structure (first)")
    gmx_rms(**mut_paths["step20_rmsfirst"], properties=mut_prop["step20_rmsfirst"])

    global_log.info("step21_rmsexp: Compute Root Mean Square deviation against minimized structure (exp)")
    gmx_rms(**mut_paths["step21_rmsexp"], properties=mut_prop["step21_rmsexp"])

    global_log.info("step22_rgyr: Compute Radius of Gyration to measure the protein compactness during the free MD simulation")
    gmx_rgyr(**mut_paths["step22_rgyr"], properties=mut_prop["step22_rgyr"])

    global_log.info("step23_image: Imaging the resulting trajectory")
    gmx_image(**mut_paths["step23_image"], properties=mut_prop["step23_image"])

    global_log.info("step24_dry: Removing water molecules and ions from the resulting structure")
    gmx_trjconv_str(**mut_paths["step24_dry"], properties=mut_prop["step24_dry"])
